from unittest.mock import patch

import pytest
from orchestrator.workflow import StepStatus
from pydantic_forms.exceptions import FormValidationError

from gso.products import Iptrunk
from gso.products.product_blocks.router import RouterRole
from test.workflows import assert_pp_interaction_success, extract_state, run_workflow


@pytest.fixture()
def ibgp_mesh_input_form_data(iptrunk_subscription_factory, faker):
    ip_trunk = Iptrunk.from_subscription(iptrunk_subscription_factory())

    return {"subscription_id": ip_trunk.iptrunk.iptrunk_sides[0].iptrunk_side_node.owner_subscription_id}


@pytest.mark.workflow()
@patch("gso.workflows.router.update_ibgp_mesh.provisioning_proxy.execute_playbook")
@patch("gso.workflows.router.update_ibgp_mesh.librenms_client.LibreNMSClient.add_device")
def test_update_ibgp_mesh_success(
    mock_librenms_add_device,
    mock_execute_playbook,
    ibgp_mesh_input_form_data,
    data_config_filename,
    faker,
):
    result, process_stat, step_log = run_workflow(
        "update_ibgp_mesh", [ibgp_mesh_input_form_data, {"tt_number": faker.tt_number()}]
    )

    for _ in range(5):
        result, step_log = assert_pp_interaction_success(result, process_stat, step_log)

    state = extract_state(result)

    assert mock_execute_playbook.call_count == 5
    assert mock_librenms_add_device.call_count == 1
    assert result.status == StepStatus.COMPLETE
    assert state["subscription"]["router"]["router_access_via_ts"] is False


@pytest.mark.workflow()
def test_update_ibgp_mesh_isolated_router(nokia_router_subscription_factory, data_config_filename):
    router_id = nokia_router_subscription_factory(router_role=RouterRole.P)

    exception_message = "Selected router does not terminate any active IP trunks."
    with pytest.raises(FormValidationError, match=exception_message):
        run_workflow("update_ibgp_mesh", [{"subscription_id": router_id}, {}])
